; Shifter - serial input module for BLINKENmini, LittleLights and BlinkenLeds
; input code for SNES pads
; version 0.2 date 2006-02-28
; Copyright (C) 2005-2006 Stefan Schuermans <1stein@schuermans.info>
; Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
; a BlinkenArea project - http://www.blinkenarea.org/



; output a clock impulse and delay to give SNES pads some time
;  - used: I_TMP
;  - used stack depth (without call to this function): 0
SNES_CLK_DELAY:
        sbi     PORT_CLK,BIT_CLK                ; output clock impulse
        nop
        cbi     PORT_CLK,BIT_CLK
        ldi     I_TMP,8
SNES_CLK_DELAY_LOOP:
        dec     I_TMP
        brne    SNES_CLK_DELAY_LOOP
        ret



; read SNES keys (1st half)
;  - uses: X, SNES_FLAG, I_CNT, I_TMP, I_REG0
;  - used stack depth (without call to this function): 2
IN_SNES_1:
        sbrc    SNES_FLAG,SNES_F_DATA           ; fetch new data only if SNES flag is cleared
        rjmp    IN_SNES_1_END
        ldi     XH,high(SNES_BUF)               ; begin at begin of SNES key states
        ldi     XL,low(SNES_BUF)
                                                ; strobe impulse was already output during last output of b/w pic 7
        ldi     I_CNT,3                         ; 3 times 2 keys
IN_SNES_1_LOOP:
        in      I_TMP,PIN_DATA                  ; read SNES key state of 1st key into low part of I_REG0
        andi    I_TMP,0x0F
        mov     I_REG0,I_TMP
        rcall   SNES_CLK_DELAY
        in      I_TMP,PIN_DATA                  ; read SNES key state of 2nd key into high part of I_REG0
        andi    I_TMP,0x0F
        swap    I_TMP
        or      I_REG0,I_TMP
        st      X+,I_REG0                       ; store SNES key data of both keys
        rcall   SNES_CLK_DELAY
        dec     I_CNT                           ; next column
        brne    IN_SNES_1_LOOP
        sbr     SNES_FLAG,1<<SNES_F_2ND         ; schedule 2nd part of work
IN_SNES_1_END:
        ret



; read SNES keys (2nd half)
;  - uses: X, SNES_FLAG, I_CNT, I_TMP, I_REG0
;  - used stack depth (without call to this function): 2
IN_SNES_2:
        sbrs    SNES_FLAG,SNES_F_2ND            ; do 2nd part of work only if requested (by 1st half)
        rjmp    IN_SNES_2_END
        ldi     XH,high(SNES_BUF+3)             ; begin at middle of SNES key states
        ldi     XL,low(SNES_BUF+3)
        ldi     I_CNT,3                         ; 3 times 2 keys
IN_SNES_2_LOOP:
        in      I_TMP,PIN_DATA                  ; read SNES key state of 1st key into low part of I_REG0
        andi    I_TMP,0x0F
        mov     I_REG0,I_TMP
        rcall   SNES_CLK_DELAY
        in      I_TMP,PIN_DATA                  ; read SNES key state of 2nd key into high part of I_REG0
        andi    I_TMP,0x0F
        swap    I_TMP
        or      I_REG0,I_TMP
        st      X+,I_REG0                       ; store SNES key data of both keys
        rcall   SNES_CLK_DELAY
        dec     I_CNT                           ; next column
        brne    IN_SNES_2_LOOP
        cbr     SNES_FLAG,1<<SNES_F_2ND         ; 2nd part of work is done
        sbr     SNES_FLAG,1<<SNES_F_DATA        ; new SNES key data is available now
IN_SNES_2_END:
        ret
